use snafu::{location, Location};

use crate::{
    bubba::{
        compiler::{compile_expression, CThonk, Context, Result, EMPTY},
        instr::Instruction,
        value::Value,
    },
    lu_dog::ValueType,
    new_ref, s_read, NewRef, RefType, SarzakStorePtr, Span, POP_CLR,
};

#[cfg_attr(not(test), tracing::instrument(skip(thonk, context)))]
pub(in crate::bubba::compiler) fn compile_list_element(
    element: &SarzakStorePtr,
    thonk: &mut CThonk,
    context: &mut Context,
) -> Result<Option<ValueType>> {
    tracing::debug!(target: "instr", "{}", POP_CLR.paint("compile_list_element"));

    let lu_dog = context.lu_dog_heel().clone();
    let lu_dog = s_read!(lu_dog);

    let element = lu_dog.exhume_list_element(element).unwrap();
    let element = s_read!(element);
    let expr = element.r55_expression(&lu_dog)[0].clone();

    compile_expression(&expr, thonk, context)?;

    Ok(None)
}

/// Compile a List Expression
///
/// Note that we can't do too much here in the compiler. We can evaluate the
/// list elements and push them on the stack, but we don't have a way of building
/// the list. That has to be taken care of at runtime.
#[cfg_attr(not(test), tracing::instrument(skip(thonk, context)))]
pub(in crate::bubba::compiler) fn compile_list_expression(
    list: &SarzakStorePtr,
    thonk: &mut CThonk,
    context: &mut Context,
    entry_span: Span,
) -> Result<Option<ValueType>> {
    tracing::debug!(target: "instr", "{}", POP_CLR.paint("compile_list_expression"));

    let lu_dog = context.lu_dog_heel().clone();
    let lu_dog = s_read!(lu_dog);

    let list = lu_dog.exhume_list_expression(list).unwrap();
    let list = s_read!(list);
    if let Some(ref element) = list.elements {
        let mut size = 1;
        let element = lu_dog.exhume_list_element(element).unwrap();
        let element = s_read!(element);
        let expr = element.r15_expression(&lu_dog)[0].clone();

        compile_expression(&expr, thonk, context)?;

        let mut next = element.next;
        while let Some(ref id) = next {
            let element = lu_dog.exhume_list_element(id).unwrap();
            let element = s_read!(element);
            let expr = element.r15_expression(&lu_dog)[0].clone();

            compile_expression(&expr, thonk, context)?;
            size += 1;

            next = element.next;
        }

        let expr = &list.r15_expression(&lu_dog)[0];
        let ty = &s_read!(expr).r11_x_value(&lu_dog)[0];
        let ty = s_read!(ty).r24_value_type(&lu_dog)[0].clone();
        let ty = s_read!(ty).clone();
        thonk.insert_instruction(Instruction::Push(Value::ValueType(ty)), location!());

        thonk.insert_instruction(Instruction::NewList(size), location!());
    } else {
        let empty = context.get_type(EMPTY).unwrap().clone();
        let ty = Value::List {
            ty: new_ref!(ValueType, empty),
            inner: new_ref!(Vec<RefType<Value>>, vec![]),
        }
        .get_value_type(context);
        let ty = s_read!(ty).clone();
        thonk.insert_instruction(Instruction::Push(Value::ValueType(ty)), location!());
        thonk.insert_instruction_with_span(Instruction::NewList(0), entry_span, location!());
    }

    Ok(None)
}
